from feedgen.feed import FeedGenerator
from flask import url_for

from mastodon_toot_follower import path_util
from mastodon_toot_follower.app import App
from mastodon_toot_follower.conversation.update import Update as ConversationUpdate


class Conversation:
    def __init__(self, app: App, mastodon_instance: str, toot_id: str, seed: str = ''):
        self.app = app
        self.mastodon = self.app.get_mastodon(mastodon_instance)

        self.toot_id = toot_id
        self.toot = self.mastodon.status(id=self.toot_id)

        if len(seed) > 0:
            self.file = self.app.toot_cache_dir().joinpath(f'{path_util.escape(self.toot["uri"])}_{seed}.json')
        else:
            self.file = self.app.toot_cache_dir().joinpath(f'{path_util.escape(self.toot["uri"])}.json')

        # List of replies to initial toot.
        self.replies = self.mastodon.status_context(id=self.toot_id)['descendants']

        self.previous_updates: list[ConversationUpdate] = ConversationUpdate.load(self.file)
        self.new_updates = ConversationUpdate.get_new_updates(self.previous_updates, self.toots())

        self._changes_saved = False

    def as_feed(self, feed_url) -> FeedGenerator:
        """
        Returns a FeedGenerator object "fg".

        atomfeed = fg.atom_str(pretty=True)  # Get the ATOM feed as string
        rssfeed = fg.rss_str(pretty=True)  # Get the RSS feed as string
        """

        fg = FeedGenerator()
        fg.id(self.toot['uri'])  # Unique feed ID.
        fg.title(f'MastodonConversation{self.toot_id}')
        fg.author({'name': self.toot['account']['acct']})
        fg.link(href=self.toot['uri'], rel='alternate')  # Webpage.
        fg.logo(url_for('static', filename='favicon.png'))
        fg.subtitle('RSS feed of a conversation on Mastodon.')
        fg.link(href=feed_url, rel='self')  # URL of the feed itself.
        fg.language('en')

        update: ConversationUpdate
        for update in self.updates():
            fe = fg.add_entry()
            fe.id(update.dict['uri'])
            acct = update.dict['acct']
            if update.dict['reason'] == 'new':
                fe.title(f'Reply by {acct}')
                fe.published(update.dict['created_at'])
            else:
                fe.title(f'Edit by {acct}')
                fe.published(update.dict['edited_at'])
            fe.link(href=update.dict['uri'])
            fe.content(update.dict['content'])

        return fg

    def conversation_length(self) -> int:
        return len(self.toots())

    def toots(self) -> list:
        """
        :return: List of initial toot and it's replies.
        """
        return [self.toot] + self.replies

    def updates(self) -> list[ConversationUpdate]:
        return self.previous_updates + self.new_updates

    def save_changes(self) -> None:
        if self._changes_saved:
            return
        ConversationUpdate.save(self.updates(), self.file)
        self._changes_saved = True
